@@ -22,6 +22,7 @@
 import base64
 import builtins
 import binascii
+import datetime
 import errno
 import io
 import os
@@ -34,7 +35,8 @@ from samba.dcerpc import drsblobs, security, gmsa
 from samba.ndr import ndr_unpack
 from samba.netcmd import Command, CommandError
 from samba.samdb import SamDB
-
+from samba.nt_time import timedelta_from_nt_time_delta, nt_time_from_datetime
+from samba.gkdi import MAX_CLOCK_SKEW
 
 # python[3]-gpgme is abandoned since ubuntu 1804 and debian 9
 # have to use python[3]-gpg instead
@@ -102,6 +104,8 @@ virtual_attributes = {
     "unicodePwd": {
         "flags": ldb.ATTR_FLAG_FORCE_BASE64_LDIF,
     },
+    "virtualManagedPasswordQueryTime": {
+    },
 }
 
 
@@ -110,7 +114,10 @@ def get_crypt_value(alg, utf8pw, rounds=0):
         "5": {"length": 43},
         "6": {"length": 86},
     }
-    assert alg in algs
+    if alg not in algs:
+        raise ValueError(f"invalid algorithm code: {alg}"
+                         f"(expected one of {','.join(algs.keys())})")
+
     salt = os.urandom(16)
     # The salt needs to be in [A-Za-z0-9./]
     # base64 is close enough and as we had 16
@@ -369,10 +376,28 @@ class GetPasswordCommand(Command):
             managed_password = obj["msDS-ManagedPassword"][0]
             unpacked_managed_password = ndr_unpack(gmsa.MANAGEDPASSWORD_BLOB,
                                                    managed_password)
-            calculated["Primary:CLEARTEXT"] = \
-                unpacked_managed_password.passwords.current
             calculated["OLDCLEARTEXT"] = \
                 unpacked_managed_password.passwords.previous
+            query_interval = unpacked_managed_password.passwords.query_interval
+            calculated["GMSA:query_interval"] = \
+                query_interval
+
+            query_time_datetime = \
+                timedelta_from_nt_time_delta(query_interval) + datetime.datetime.now(tz=datetime.timezone.utc)
+
+            query_time_nttime = nt_time_from_datetime(query_time_datetime)
+
+            calculated["GMSA:query_time"] = query_time_nttime
+
+            # This password is useful for a keytab, but not for
+            # authentication, so don't show or provide it as the new password
+            # just yet
+            if calculated["GMSA:query_interval"] <= MAX_CLOCK_SKEW:
+                calculated["Primary:CLEARTEXT"] = \
+                    unpacked_managed_password.passwords.previous
+            else:
+                calculated["Primary:CLEARTEXT"] = \
+                    unpacked_managed_password.passwords.current
 
         account_name = str(obj["sAMAccountName"][0])
         if "userPrincipalName" in obj:
@@ -467,7 +492,8 @@ class GetPasswordCommand(Command):
                         decrypted = tmp.get_nt_hash()
                         current_hash = unicodePwd
                     elif aes256_key is not None and kerberos_salt is not None:
-                        decrypted = tmp.get_aes256_key(kerberos_salt)
+                        tmp.set_kerberos_salt_principal(kerberos_salt)
+                        decrypted = tmp.get_kerberos_key(credentials.ENCTYPE_AES256_CTS_HMAC_SHA1_96)
                         current_hash = aes256_key.value
 
                     if current_hash is not None and current_hash == decrypted:
@@ -770,6 +796,10 @@ class GetPasswordCommand(Command):
                 v = get_wDigest(i, primary_wdigest, account_name, account_upn, domain, dns_domain)
                 if v is None:
                     continue
+            elif a == "virtualManagedPasswordQueryTime":
+                if "GMSA:query_time" not in calculated:
+                    continue
+                v = str(calculated["GMSA:query_time"])
             else:
                 continue
             obj[a] = ldb.MessageElement(v, ldb.FLAG_MOD_REPLACE, vattr["raw_attr"])
@@ -840,10 +870,14 @@ class GetPasswordCommand(Command):
                     continue
                 if ra["vformat"] != fm:
                     continue
+
                 srcattr = get_src_attrname(ra["attr"])
+                if srcattr is not None:
+                    an = "%s;format=%s" % (srcattr, fm)
+                else:
+                    srcattr = an = get_src_attrname(ra["raw_attr"])
                 if srcattr is None:
                     continue
-                an = "%s;format=%s" % (srcattr, fm)
                 if an in generated_formats:
                     continue
                 generated_formats[an] = fm
